import { Annotation, PlottableModel, Transform, TransformType } from './4d-objects';
import { Observable } from 'rxjs';
import { VasatModelClass } from 'vasat';
import { PlottableReference, Renderer } from './renderer';
/**
 * <p>Interface for the helper classes. Currently there are three maps that are supported (<a href="https://cesiumjs.org/tutorials/">Cesium</a>, <a href="https://developers.google.com/maps/documentation/javascript/tutorial">Google</a>, <a href="https://leafletjs.com/">Leaflet</a>).</p>
 */
export declare class MapHelper {
    protected plotCache: {
        [tid: string]: PlottableReference;
    };
    protected renderers: {
        [className: string]: {
            [filter: string]: Renderer<any>;
        };
    };
    _debug_1: string;
    _debug_2: string;
    _log(id?: string, data?: any): void;
    /**
     * <p>Removes all models/images from the map.</p>
     */
    clearPlottables(): void;
    /**
     * <p>Adds specified model/images to the map.</p>
     * <p>See [[Transform]] for details on compatibility.</p>
     * @param t Accepts [[Transform]] and [[Annotation]] objects.
     * @param args
     * @deprecated use setPlottable
     */
    addPlottable(t: PlottableModel<any>, args?: any): Observable<PlottableReference>;
    /**
     * <p>Adds or Updates specified model/images to the map.</p>
     * <p>See [[Transform]] for details on compatibility.</p>
     * @param t Accepts [[Transform]] and [[Annotation]] objects.
     * @param args
     */
    setPlottable(p: PlottableModel<any>, args?: any): Observable<PlottableReference>;
    /**
     * <p>Removes specified model object from map.</p>
     * @param t The [[PlottableModel]] object to be removed.
     */
    removePlottable(p: PlottableModel<any>): void;
    /**
     * <p>Pans the camera to specified model object on the map.</p>
     * @param t The [[PlottableModel]] object that you wish to pan the camera to.
     * @param duration Time taken to pan camera to destination in milliseconds.
     * @param args
     */
    sendCameraTo(t: PlottableModel<any>, duration?: number, args?: any): Observable<any>;
    /**
     * <p>Returns the respective Reference object of a given [[PlottableModel]] object that was plotted on the map.</p>
     * @param p [[PlottableModel]] object
     */
    getMapObjectFor(p: PlottableModel<any>): PlottableReference;
    /**
     * <p>Returns the authenticated url of the image/model of selected [[PlottableModel]].</p>
     * @param p The [[PlottableModel]] object
     * @param file Name of file.
     * @param tier Applicable for Cesium only. See [[FourdCesiumHelper.getAuthenticatedUrl]] for more details.
     */
    getAuthenticatedUrl(p: PlottableModel<any>, file: string, tier: number): Promise<any>;
    /**
     * Change the transparency for a transform being displayed on this map. Transform must have been added using addPlottable
     * @param t the transform object
     * @param opacity zero - 1 as percent
     */
    changeOpacity(t: Transform, opacity: number): void;
    /**
     * use whatever this implementations click event is to resolve the item currently clicked
     */
    resolveItem(clickEvent: any): PlottableModel<any>;
    registerRenderer(clazz: VasatModelClass<Annotation | Transform>, category: string | TransformType, renderer: Renderer<Annotation | Transform>): void;
    destroy(): void;
    private classNameForType;
    private getRenderer;
}
