import { Observable, Subject } from 'rxjs';
import { Vasat } from 'vasat';
/**
 * <p>Processing Event. Includes </p>
 */
export declare class ServerProcess {
    uid: string;
    dataset: any;
    name: string;
    status: string;
    info: any;
    progress: number;
    error: any;
    uploader: ChunkedFileUploader;
    update(a: any): ServerProcess;
}
/**
 * <p>Interface that outlines common properties of an upload result.</p>
 */
export interface UploadRes {
    finished: boolean;
    error: any;
    status?: number;
    aborted: any;
    chunk: ChunkedFileUploader;
}
/**
 * <p>Breaks files into manageable chunks for upload.</p>
 */
export declare class ChunkedFileUploader {
    private V;
    private url;
    loaded: number;
    step: number;
    total: number;
    start: number;
    progress: number;
    progressSubject: Subject<number>;
    progressObservable: Observable<number>;
    cnx: boolean;
    fileRef: File;
    retrysB4Fail: number;
    constructor(V: Vasat, url: string, chunkSize?: number);
    /**
     * <p>Cancels the current XMLHttpRequest.</p>
     */
    cancel(): void;
    currentXHR: XMLHttpRequest;
    /**
     * <p>Uploads the a file by fragmenting it to smaller pieces depending on the given parameters.</p>
     * @param file File to upload.
     * @param startOffset Start of offset.
     * @param seqOffset The sequence number, where 0 is first.
     */
    uploadObservable(file: any, startOffset?: number, seqOffset?: number): Observable<UploadRes>;
}
