import { Observable } from 'rxjs';
import { FourDService } from './4d-mapper.service';
import { Transform, PlottableModel } from './4d-objects';
import { MapHelper } from './4d-helper';
/**
 * <p>Helper class for <a href="https://cesiumjs.org/tutorials/">Cesium</a>. Contains methods to zoom to a location and add/remove a model to the map.</p>
 * <p>You can play with a(n) (<a href="https://stackblitz.com/edit/4dmapper-sdk">angular</a> or <a href="https://stackblitz.com/edit/react-demo-cesium">react</a>) demo on stackblitz.</p>
 */
export declare class FourdCesiumHelper extends MapHelper {
    fourdD: FourDService;
    viewer: any;
    private _originalTerrainProvider;
    dziDatasource: any;
    annotationDataSource: any;
    annotationLabels: any;
    constructor(fourdD: FourDService, viewer: any, cesiumObject?: any);
    destroy(): void;
    /**
     * <p>Returns the respective CesiumPlottableReference object of a given [[PlottableModel]] object</p>
     * @param p [[PlottableModel]] object
     * @deprecated use #getMapObjectFor instead
     */
    getCesiumObjectFor(p: PlottableModel<any>): any;
    clearPlottables(): void;
    sendCameraTo(t: PlottableModel<any>, duration?: number, args?: any): Observable<any>;
    /**
     * <p>If the server supports it, get a token suffix that may be a remote storage server (eg S3 or Cloudfront URL)
     * Cesium can query resources directly rather than hit application server</p>
     * @param p [[PlottableModel]] Object
     * @param file Name of file
     * @param tier Applicable when Cloudfront is set up. Cesium allows request from one url at a time, so by providing a 'tier' which represents a different url, we can better conduct traffic. 0: default/Terrain Tiles, 1: Tile Map Service (tms), 2: 3D Tiles
     */
    getAuthenticatedUrl(p: PlottableModel<any>, file: string, tier?: number): Promise<string>;
    changeOpacity(t: Transform, opacity: number): void;
    resolveItem(movement: any): any;
}
