import { MapHelper } from './4d-helper';
import { Observable } from 'rxjs';
import { PlottableModel, Transform } from './4d-objects';
import { PlottableReference } from './renderer';
/**
 * <p>Helper class for <a href="https://developers.google.com/maps/documentation/javascript/tutorial">Google Maps</a>. Contains methods to zoom to a location and add/remove a model to the map.</p>
 * <p>You can play with a(n) (<a href="https://stackblitz.com/edit/angular-demo-google">angular</a> or <a href="https://stackblitz.com/edit/react-demo-google">react</a>) demo on stackblitz.</p>
 */
export declare class GoogleHelper extends MapHelper {
    map: any;
    mapRef: any;
    constructor(map: any, mapRef: any);
    /**
     * <p>Retrieves the url for tiles.</p>
     */
    getAuthenticatedUrl(p: PlottableModel<any>, file: string, tier?: number): Promise<string[]>;
    /**
     * <p>Create a map type that gets custom tiles and overlay over the google map (<a href="https://developers.google.com/maps/documentation/javascript/maptypes#OverlayMapTypes">docs</a>).</p>
     */
    setPlottable(p: PlottableModel<any>, args?: any): Observable<PlottableReference>;
    clearPlottables(): void;
    getMapObjectFor(p: PlottableModel<any>): any;
    /**
     * <p>Get the bounds from SW and NE lat and long given [[PlottableModel]] object.</p>
     */
    getBounds(p: PlottableModel<any>): any;
    /**
     * <p>Pan the camera to specified bounds.</p>
     */
    sendCameraTo(t: PlottableModel<any>, duration?: number, args?: any): Observable<any>;
    /**
     * <p>Normalizes the coords that tiles repeat across the x axis (horizontally) (taken from <a href="https://developers.google.com/maps/documentation/javascript/maptypes#ImageMapTypes">GoogleMaps</a>).</p>
     */
    getNormalizedCoord(coord: any, zoom: any): {
        x: any;
        y: any;
    };
    changeOpacity(t: Transform, opacity?: number): void;
    resolveItem(movement: any): PlottableModel<any>;
}
